#######################################################################################################
# Copyright Orchid Systems 2020
# =============================================
# Desc: View script for making optional field mandatory.
# Modify global listOne, listTwo or listThree and mandatoryCondition functions as required
# COLOR, LEADTIME, MANFACTURER for Sellable items
# COLOR for items in Category A1
# Attach to IC0310
#######################################################################################################
# Disclaimer:
# This code is to be used with Orchid Systems' Extender module only.
# Orchid Systems provides the code on an as is basis having been developed and tested in-house 
# using a standard Sage 300 implementation with Sage sample data.
# If this code is deployed in a live production environment it is the responsibility of the End User
# and the Sage Business Partner to ensure that is is operating as required.
#######################################################################################################

from accpac import *

# onOpen is called when the view is opened.
# Return Continue to enable the macro.
# Return 0 to disable the macro.
# If you return anything else then the view will not load.

# global optional field lists.
listOne = ["COLOR", "LEADTIME", "MANFACTURER"]
listTwo = ["COLOR"]
listThree = []

# global views.
icitemo = None

def onOpen():
	
	if rotoID != "IC0310":
		return 0 # don't do anything since the view is not I/C Items.

	return Continue
	
def onBeforeCompose(e):
	global icitemo
	
	# Get item optional fields view if composed to the ic items master view.
	if e.views[7] is None:
		icitemo = None
	else:
		icitemo = e.views[7]
	
	return Continue

def onBeforeUpdate():
	# validation function returns either Continue or something else which would indicate an error.
	return VerifyOptionalFields()

def onBeforeInsert():
	# validation function returns either Continue or something else which would indicate an error.
	return VerifyOptionalFields()

### Business Logic ####

def VerifyOptionalFields():
	rvspyTrace("VerifyOptionalFields")

	# Returns either Continue or 1 (indicating an optional field failed to meet a non-blank requirement)
	
	# default return value:
	bContinue = Continue

	# if the condition is true, check optional fields are filled in:
	if mandatoryConditionOne() == True:
		if _verifyOptionalFields(listOne) == False:
			bContinue = 1	# There's one or more invalid optional fields.

	if mandatoryConditionTwo() == True:
		if _verifyOptionalFields(listTwo) == False:
			bContinue = 1	# There's one or more invalid optional fields.

	if mandatoryConditionThree() == True:
		if _verifyOptionalFields(listThree) == False:
			bContinue = 1	# There's one or more invalid optional fields.
		
	# add more conditions as necessary or other validation requirements.
		
	return bContinue

# returns TRUE or FALSE.
def mandatoryConditionOne():
	rvspyTrace("mandatoryConditionOne")
	
	# Is the item sellable?
	isSellable = int(me.get("SELLABLE"))
	if isSellable == 1:
		# True, item is sellable.
		return True # Mandatory condition for testing is True.
	else:
		return False # Mandatory condition for testing is False. 
		
# returns TRUE or FALSE.
def mandatoryConditionTwo():
	rvspyTrace("mandatoryConditionTwo")
	
	# Is the item in category A1?
	sItemCategory = str(me.get("CATEGORY"))
	if sItemCategory == "A1":
		# True, item is sellable.
		return True # Mandatory condition for testing is True.
	else:
		return False # Mandatory condition for testing is False. 
	
# returns TRUE or FALSE.
def mandatoryConditionThree():
	rvspyTrace("mandatoryConditionThree")
	
	return False	# Set to False to indication the condition for testin is not True.
	
	
def _verifyOptionalFields(optFldList):

	global icitemo
	
	# returns False if one or more optional fields failed the validation.
	# return True if nothing to test or all the optional fields passed the validation.
	
	if icitemo == None:
		# the ic item view is not composed to optional fields.
		# cannot validate so return True to skip the validation.
		return True
	
	# if the list of optional fields is blank, do nothing.
	if len(optFldList) == 0:
		return True		# nothing to test.
		
	# default return value.
	# As soon as an optional field fails the test for whatever reason, this switches to False and stays False.
	bValid = True
		
	# otherwise continue checking the optional fields:
	for optFld in optFldList:
		#showMessageBox("test optional field: " + optFld)
		
		# check if blank.
		if icitemo.recordClear() == 0:
			item = icitemo.get("ITEMNO").strip()
			if item == "":
				# the record clear wiped out all the key and did not initialize the part inherited from ICITEM.
				# we need to fill it in manually.
				icitemo.put("ITEMNO", me.get("ITEMNO").strip())
		
			r = icitemo.put("OPTFIELD", optFld)
			if r != 0:
				# something went wrong. The put failed. There will be an error on the message stack.
				# However, we let the UI (screen) display it.
				bValid = False
			else:
				if icitemo.read() != 0:
					# optional field record does not exist.
					msg = "Mandatory Optional Field [{}] record not found. ".format(optFld)
					error(msg)
					bValid = False
				else:
					# the optional field record was found. Check if the optional field value is blank.
					optVal = str(icitemo.get("VALUE")).strip()
					if optVal == "":
						msg = "Invalid Entry: Mandatory Optional Field [{}] value is blank.".format(optFld)
						error(msg)
				
	return bValid
	
